# 
#  This file is part of Herschel Common Science System (HCSS).
#  Copyright 2001-2013 Herschel Science Ground Segment Consortium
# 
#  HCSS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation, either version 3 of
#  the License, or (at your option) any later version.
# 
#  HCSS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
# 
#  You should have received a copy of the GNU Lesser General
#  Public License along with HCSS.
#  If not, see <http://www.gnu.org/licenses/>.
# 
#
# This ipipe script processes scan map and mini-scan map observations
# containing mostly point-like or relatively small extended sources. 
# starting from Level 1 (calibrated data cubes in Jy/detector pixel)
# and combinining several (crossed) observations.
# This script makes uses of an iterative high-pass filtering of the
# timelines to remove the 1/f noise by masking the source(s)
# This processing is not adequate for extended sources 
# as it removes large-scale structures, which cannot be
# properly protected (masked) from the filtering.
#
# nifty add-ons:
#  - for faint sources, there is the possibility to inject the ra, dec of the source
# for the mask and the photometry such that the correct source is properly masked
# and the aperture is centered at the required position
#
# The source fitting function is fully described in the simple aperture photometry script 
# that can be found in the useful scripts 
#
# Bruno Altieri, Vanessa Doublier, Javier Gracia
# September 2013, for Hipe 12

import os
from java.util import ArrayList
from herschel.ia.toolbox.image import MosaicTask
from herschel.pacs.spg.phot import MaskFromCatalogueTask
from herschel.pacs.spg.pipeline import *

"""
Defining the object list: 
 1- group 70mu or 100mu
 2- for the red maps: all 70/100mu can be listed together
"""
object = "2003AZ84"

direc = "/Users/pkisscs/2003AZ84/"
fileRoot = direc
saveIntermediateSteps = False

#obsids=[1342205222,1342205223,1342205224,1342205225,1342205152]
obsids=[1342205152]

for obsid in obsids:
	path0 = fileRoot+str(obsid)
	if not os.path.isdir(path0):
		os.mkdir(fileRoot+str(obsid))

"""
Is it a solar system object?
"""
sso = False

"""
Define the output directory for save frames and maps
either: define manually
or define it as the directory from which you start hipe
"""

"""
Settings for the processing
"""
calculateRaDec = False  # if True, processing is faster, but it requires 3 times more memory
doIIndLevelDeg = True # if True, a finer deglitching (nsigma=15) than SPG level 1 (nsigma=30) is performed
doPhotometry = False  # if True, does the photometry on the source(s) at the end
doSourceFit = False #  iF True, fit the source to get the centre for the aperture photometry
#                      other if false rely on source coordinates in metadata
fromAFile = False # load the source coordinates from a catalogue.
tfile = direc + "myTargets.txt"

"""
Define the turnaround removal:
1- upper and lower limits
2- percent above/below scan speed
"""
lowScanSpeed  = 15
highScanSpeed = 25
limits = 10.
"""
Get the observations from the local store or the HSA and save them in a list
"""
cameras = ["red","blue"]

for obsid in obsids: 
 obs=getObservation(obsid, useHsa=True, instrument="PACS")
 calTree = getCalTree(obs=obs)

 for camera in cameras:
  if camera == "blue":
   if obs.meta["blue"].value == 'blue2': 
    outpixsz = 1.4  # map pixel size (arcsec)
    filter = 'green'
    hpfradius = 15
   else: 
    outpixsz = 1.1
    filter = 'blue'
    hpfradius = 10
   pixfrac = 0.1   # bolometer pixel drop size
  else:
   outpixsz = 2.1  # map pixel size (arcsec)
   pixfrac = 0.1   # bolometer pixel drop size
   filter = 'red'
   hpfradius = 25
  
  print "Start processing: ", obsid
  if camera == "blue":
     frames = obs.level1.refs["HPPAVGB"].product.refs[0].product
  else:
     frames = obs.level1.refs["HPPAVGR"].product.refs[0].product
  pp = obs.auxiliary.pointing
  frames = photAddInstantPointing(frames, pp, calTree=calTree, copy=1, absShift=0.0)
  # Calculate the coordinates of every pixel with time 
  if calculateRaDec: 
     frames = photAssignRaDec(frames)
  # Save the frames variable at its current state so we can use it later
  save(fileRoot + "_" + str(obsid) + "_framesWithoutHPF.ser", "frames")
  #
  # First pass with highpass filter 
  #
  frames = highpassFilter(frames, hpfradius, interpolateMaskedValues=True)
  #frames = filterOnScanSpeed(frames, lowScanSpeed=lowScanSpeed, highScanSpeed=highScanSpeed)
  frames = filterOnScanSpeed(frames, limit=limits)
  map, mi = photProject(frames, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree)
  #d = Display(map, title="HighpassFilter task result on " + str(obsid) + " (without source mask)")
  if saveIntermediateSteps :
    simpleFitsWriter(map, fileRoot + "_" + str(obsid) + "_map_withoutMask_firstStep.fits")
  # Obtain an source mask from this initial map
  med = MEDIAN(map.coverage[map.coverage.where(map.coverage > 0.)])
  index = map.image.where((map.coverage > med) & (ABS(map.image) < 1e-2))
  signal_stdev = STDDEV(map.image[index])
  threshold = 3.0*signal_stdev
  print "Threshold for the source mask = ", threshold
  mask = map.copy()
  mask.image[mask.image.where(map.image > threshold)] = 1.0
  mask.image[mask.image.where(map.image < threshold)] = 0.0
  mask.image[mask.image.where(map.coverage < 0.5*med)] = 0.0
  #d = Display(mask, title="Source mask for " + str(obsid))
  if saveIntermediateSteps :
    simpleFitsWriter(mask, fileRoot + "_" + str(obsid) + "_mask_firstStep.fits")
  #
  # Second pass with masked highpass filtering
  #
  restore(fileRoot + "_" + str(obsid) + "_framesWithoutHPF.ser")
  # Introduce the mask in the frames and apply the highpass filtering
  frames, outMask = photReadMaskFromImage(frames, si=mask, maskname="HighpassMask", extendedMasking=True, calTree=calTree)
  frames = highpassFilter(frames, hpfradius, maskname="HighpassMask", interpolateMaskedValues=True)
  #frames = filterOnScanSpeed(frames, lowScanSpeed=lowScanSpeed, highScanSpeed=highScanSpeed)
  frames = filterOnScanSpeed(frames, limit=limits)
  map, mi = photProject(frames, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree)
  #d = Display(map, title="HighpassFilter task result on " + str(obsid) + " (with a source mask)")
  simpleFitsWriter(map, fileRoot + "_" + str(obsid) + "_map_firstStep.fits") 
  
  # Remove variables that are not needed anymore
  #del frames, map, mi, d, med, index, signal_stdev, threshold, mask, outMask 
  
  
  mosaic = simpleFitsReader(file=fileRoot + "_" + str(obsid) + "_map_firstStep.fits")
  #d = Display(mosaic, title="Mosaic map")
  if saveIntermediateSteps :
   simpleFitsWriter(mosaic, fileRoot + "_mosaic_firstStep.fits")
  
  # Obtain a source mask using the mosaic map
  med = MEDIAN(mosaic.coverage[mosaic.coverage.where(mosaic.coverage > 0.)])
  index = mosaic.image.where((mosaic.coverage > med) & (ABS(mosaic.image) < 1e-2))
  signal_stdev = STDDEV(mosaic.image[index])
  threshold = 2.0*signal_stdev
  print "Threshold for the source mask = ", threshold
  mosaicMask = mosaic.copy()
  mosaicMask.image[mosaicMask.image.where(mosaic.image > threshold)] = 1.0
  mosaicMask.image[mosaicMask.image.where(mosaic.image < threshold)] = 0.0
  mosaicMask.image[mosaicMask.image.where(mosaic.coverage < 0.5*med)] = 0.0
  #d = Display(mosaicMask, title="Source mask obtained using the mosaic map")
  if saveIntermediateSteps :
   simpleFitsWriter(mosaicMask, fileRoot + "_mosaicMask_firstStep.fits")
  
  # Remove variables that are not needed anymore
  del mosaic, med, index, signal_stdev, threshold 
  
  print "Start processing: ", obsid
  #restore(fileRoot + "_" + str(obsid + "_framesWithoutHPF.ser")
  restore(fileRoot + "_" + str(obsid) + "_framesWithoutHPF.ser")
  frames, outMask = photReadMaskFromImage(frames, si=mask, maskname="HighpassMask", extendedMasking=True, calTree=calTree)
  frames = highpassFilter(frames, hpfradius, maskname="HighpassMask", interpolateMaskedValues=True)
  # Apply 2nd level deglitching after highpass filtering
  if(doIIndLevelDeg):
    s = Sigclip(10, 15) # nsigma=15 (second parameter) for sigma clipping
    s.behavior = Sigclip.CLIP
    s.outliers = Sigclip.BOTH_OUTLIERS
    s.mode = Sigclip.MEDIAN
    mapDeglitch(frames, algo=s, deglitchvector="timeordered", calTree=calTree)
    del s
  #frames = filterOnScanSpeed(frames, lowScanSpeed=lowScanSpeed, highScanSpeed=highScanSpeed)
  frames = filterOnScanSpeed(frames, limit=limits)
  map, mi = photProject(frames, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree)
  #d = Display(map, title="HighpassFilter task result on " + str(obsid) + " (with the mosaic source mask)")
  simpleFitsWriter(map, fileRoot + "_" + str(obsid) + "_map_secondStep.fits")
  # Save the glitch mask into the non highpass filtered frames and save them again for the next step
  glitchmaskMeta = frames.meta["Glitchmask"]
  glitchmask = frames.getMask(glitchmaskMeta.value)
  restore(fileRoot + "_" + str(obsid) + "_framesWithoutHPF.ser")
  frames.setMask(glitchmaskMeta.value, glitchmask)
  save(fileRoot + "_" + str(obsid) + "_framesWithoutHPF.ser", "frames")
  
  # Remove variables that are not needed anymore
  del frames, outMask, map, mi, glitchmaskMeta,  glitchmask
  
  mosaic = simpleFitsReader(file=fileRoot + "_" + str(obsid) + "_map_secondStep.fits")
  #d = Display(mosaic, title="Mosaic map")
  if saveIntermediateSteps :
   simpleFitsWriter(mosaic, fileRoot + "_mosaic_secondStep.fits")
  
  # Obtain a new source mask using the mosaic map
  med = MEDIAN(mosaic.coverage[mosaic.coverage.where(mosaic.coverage > 0.)])
  index = mosaic.image.where((mosaic.coverage > med) & (ABS(mosaic.image) < 1e-2))
  signal_stdev = STDDEV(mosaic.image[index])
  threshold = 2.0*signal_stdev
  print "Threshold for the source mask = ", threshold
  mosaicMask = mosaic.copy()
  mosaicMask.image[mosaicMask.image.where(mosaic.image > threshold)] = 1.0
  mosaicMask.image[mosaicMask.image.where(mosaic.image < threshold)] = 0.0
  mosaicMask.image[mosaicMask.image.where(mosaic.coverage < 0.5*med)] = 0.0
  #d = Display(mosaicMask, title="Source mask obtained using the mosaic map")
  if saveIntermediateSteps :
   simpleFitsWriter(mosaicMask, fileRoot + "_mosaicMask_secondStep.fits")
  
  # Remove variables that are not needed anymore
  del med, index, signal_stdev, threshold 
  
  # Get source Ra and Dec from the metadata
  tlist = String1d(1, obs.meta["object"].value)
  rasource  = Double1d(1, obs.meta["ra"].value)
  decsource = Double1d(1, obs.meta["dec"].value)
  
  
  # Combine the the targets mask with the mosaic source mask
  radius = 20.
  combinedMask = mosaicMask.copy()
  #combinedMask.image = combinedMask.image*0. # uncomment if source list mask only
  mfc = MaskFromCatalogueTask()
  combinedMask = mfc(combinedMask, rasource, decsource, Double1d(len(rasource), radius), copy=1)
  #d = Display(combinedMask, title="Combined mask")
  if saveIntermediateSteps :
   simpleFitsWriter(combinedMask, fileRoot + "_finalMosaicMask.fits")
  
  # Remove variables that are not needed anymore
  del mosaic, radius, mfc, mosaicMask
  
  nms=[2]
  bins=[2]
  
  print "Start processing: ", obsid
  restore(fileRoot + "_" + str(obsid) + "_framesWithoutHPF.ser")
  frames, outMask = photReadMaskFromImage(frames, si=combinedMask, maskname="HighpassMask", extendedMasking=True, calTree=calTree)
  frames = highpassFilter(frames, hpfradius, maskname="HighpassMask", interpolateMaskedValues=True)
  #frames = filterOnScanSpeed(frames, lowScanSpeed=lowScanSpeed, highScanSpeed=highScanSpeed)
  frames = filterOnScanSpeed(frames, limit=limits)
  
  rep_array = frames.getStatus('Repetition')
  min_rep = MIN(rep_array)
  max_rep = MAX(rep_array)
  chopObservation = False
  if chopObservation ==True:
    for nm in nms:
     os.mkdir(direc+str(obsids[0])+'/nm'+str(nm)+'_pf0.1_'+camera)
     os.mkdir(direc+str(obsids[0])+'/nm'+str(nm)+'_pf1.0_'+camera)
     if nm == 3: bin=1
     if nm == 1: bin=1
     #if nm == 8: bin=4
  
    
     for i in range(min_rep,max_rep-bin,bin):
      min_rsel=i
      max_rsel=i+nm-1
  
      sframes = frames.select( (frames.getStatus('Repetition') >=min_rsel) .and (frames.getStatus('Repetition') <= max_rsel) )
      print '>>Selected repetitions: ', UNIQ(sframes.getStatus('Repetition'))
  
      # Make a map with the default pixfrac (=1)
      map, mi = photProject(sframes, pixfrac=1, outputPixelsize=outpixsz, calTree=calTree[0])
      #d = Display(map, title="Final map of " + str(obsid) + " (pixfrac = 1)")
      simpleFitsWriter(map, file=direc+str(obsids[0])+'/nm'+str(nm)+'_pf'+str(1.0)+'_'+camera+'/'+str(obsids[0])+'_'+camera+'_nm='+str(nm)+'_sr='+str(i)+'.fits')
      # Make a map with the user defined pixfrac
      map, mi = photProject(sframes, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree[0])
      #d = Display(map, title="Final map of " + str(obsid) + " (pixfrac = " + str(pixfrac) + ")")
      simpleFitsWriter(map, file=direc+str(obsids[0])+'/nm'+str(nm)+'_pf'+str(0.1)+'_'+camera+'/'+str(obsids[0])+'_'+camera+'_nm='+str(nm)+'_sr='+str(i)+'.fits')
  
  # Make a map with the default pixfrac (=1)
  map, mi = photProject(frames, pixfrac=1, outputPixelsize=outpixsz, calTree=calTree)
  #d = Display(map, title="Final map of " + str(obsid) + " (pixfrac = 1)")
  simpleFitsWriter(map, file=direc+str(obsids[0])+'/'+str(obsid)+'_'+camera+'_nm='+str(max_rep)+'_pf=1.0.fits')
  # Make a map with the user defined pixfrac
  map, mi = photProject(frames, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree)
  #d = Display(map, title="Final map of " + str(obsid) + " (pixfrac = " + str(pixfrac) + ")")
  simpleFitsWriter(map, file=direc+str(obsids[0])+'/'+str(obsid)+'_'+camera+'_nm='+str(max_rep)+'_pf=0.1.fits')
  
  # Remove variables that are not needed anymore
  del frames, outMask, map, mi, combinedMask
  
### END
