# 
#  This file is part of Herschel Common Science System (HCSS).
#  Copyright 2001-2013 Herschel Science Ground Segment Consortium
# 
#  HCSS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation, either version 3 of
#  the License, or (at your option) any later version.
# 
#  HCSS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
# 
#  You should have received a copy of the GNU Lesser General
#  Public License along with HCSS.
#  If not, see <http://www.gnu.org/licenses/>.
# 
#
# This ipipe script processes scan map and mini-scan map observations
# containing mostly point-like or relatively small extended sources. 
# starting from Level 1 (calibrated data cubes in Jy/detector pixel)
# and combinining several (crossed) observations.
# This script makes uses of an iterative high-pass filtering of the
# timelines to remove the 1/f noise by masking the source(s)
# This processing is not adequate for extended sources 
# as it removes large-scale structures, which cannot be
# properly protected (masked) from the filtering.
#
# nifty add-ons:
#  - for faint sources, there is the possibility to inject the ra, dec of the source
# for the mask and the photometry such that the correct source is properly masked
# and the aperture is centered at the required position
#
# The source fitting function is fully described in the simple aperture photometry script 
# that can be found in the useful scripts 
#
# Bruno Altieri, Vanessa Doublier, Javier Gracia
# September 2013, for Hipe 12

import os
from java.util import ArrayList
from herschel.ia.toolbox.image import MosaicTask
from herschel.pacs.spg.phot import MaskFromCatalogueTask
from herschel.pacs.spg.pipeline import *

"""
Defining the object list: 
 1- group 70mu or 100mu
 2- for the red maps: all 70/100mu can be listed together
"""
object = "Quaoar2"
#obsids = [1342188245,1342188246]   # alpha Boo
#obsids = [1342210582,1342210583]   # gamma Dra
#obsids = [1342204327,1342204328]   # Ceres
obsids0 = [1342189191, 1342189192]  # delta Dra A
obsids1 = [1342212499, 1342212500]  # delta Dra B
obsids2 = [1342222149, 1342222150]  # delta Dra C
obsids3 = [1342250095, 1342250096]  # delta Dra D
obsids4 = [1342257989, 1342257990]  # delta Dra E
obsids5 = [1342185446, 1342185447]  # HD 138265 A
obsids6 = [1342188841, 1342188842]  # HD 138265 B
obsids7 = [1342186163, 1342186164]  # HD 159330 A
obsids8 = [1342213585, 1342213586]  # HD 159330 B

obslist=[obsids0,obsids1, obsids2,obsids3,obsids4,obsids5,obsids6,obsids7,obsids8,obsids0,obsids1, obsids2,obsids3,obsids4,obsids5,obsids6,obsids7,obsids8]
objlist=["DDraA", "DDraB", "DDraC", "DDRaD", "DDraE", "HD138265A", "HD138265B", "HD159330A", "HD159330B", "DDraA2", "DDraB2", "DDraC2", "DDRaD2", "DDraE2", "HD138265A2", "HD138265B2", "HD159330A2", "HD159330B2"]
camlist=["red", "blue"]
hpf=[1,1,1,1,1,1,1,1,1, 0,0,0,0,0,0,0,0,0]
file = open("Photometry_results_standardstars.txt", 'w')
item1_ind=0

for item1 in obslist:
    obsids = item1
    object = objlist[item1_ind]
    hpfR = hpf[item1_ind]
    print obsids
    print object
    item1_ind=item1_ind+1

    for item2 in camlist:
        camera = item2
        print camera
        
        #camera = "blue"
        #camera = "red"
        
        """
        Is it a solar system object?
        """
        sso = False
        
        """
        Define the output directory for save frames and maps
        either: define manually
        or define it as the directory from which you start hipe
        """
        direc = "/Users/Erika/WORK/TNO/DP/" + object + "/"
        if not os.path.exists(direc):
            os.makedirs(direc)
        #direc = os.getcwd() + "/"

        saveIntermediateSteps = False

        """
        Settings for the processing
        """
        calculateRaDec = False  # if True, processing is faster, but it requires 3 times more memory
        doIIndLevelDeg = True # if True, a finer deglitching (nsigma=15) than SPG level 1 (nsigma=30) is performed
        doPhotometry = True  # if True, does the photometry on the source(s) at the end
        doSourceFit = True #  iF True, fit the source to get the centre for the aperture photometry
        #                      other if false rely on source coordinates in metadata
        fromAFile = False # load the source coordinates from a catalogue.
        tfile = direc + "myTargets.txt"
        
        """
        Define the turnaround removal:
        1- upper and lower limits
        2- percent above/below scan speed
        """
        #lowScanSpeed  = 15
        #highScanSpeed = 22
        limits = 10.
        
        """
        Processing variables set-up:
        (Note that the units of the coverage map vary with the value of pixfrac, as pixfrac**2. 
        Using pixfrac = 0.1 will reduce the coverage units by a factor 0.01 compared to the
        default (pixfrac=1). In most cases this has not a major impact on the data analysis, 
        because the coverage map is normally used to compare its value on relative terms, 
        between different pixels of the same map. However, you should keep this dependency 
        in mind if you want to compare the coverage of two maps produced with different pixfracs)
        """
        #if camera == "blue":
        #  outpixsz = 1.0  # map pixel size (arcsec)
        #  pixfrac = 0.1   # bolometer pixel drop size
        #else:
        #  outpixsz = 2.0  # map pixel size (arcsec)
        #  pixfrac = 0.1   # bolometer pixel drop size
        
        """
        Get the observations from the local store or the HSA and save them in a list
        """
        obs = [] 
        for i in range(len(obsids)):
          obs.append(getObservation(obsids[i], useHsa=True, instrument="PACS"))
        
        """
        Load the calibration trees. Either use HSA calTree or use an updated calTree 
        (version can be check using print obs[0].calibration and print (getCalTree(obs=obs))
        """
        calTree = []
        for i in range(len(obsids)):
          calTree.append(getCalTree(obs=obs[i]))
        
        """
        If we have a solar system object, we need to set the coordinates to the object
        reference system
        """
        if sso:
          for i in range(len(obsids)):
            if camera == "blue":
              frames = obs[i].level1.refs["HPPAVGB"].product
            else:
              frames = obs[i].level1.refs["HPPAVGR"].product
            pp = obs[i].auxiliary.pointing
            orbitEphem = obs[i].auxiliary.orbitEphemeris
            horizonsProduct = obs[i].auxiliary.horizons
            # Add again the pointing information to the status 
            frames = photAddInstantPointing(frames, pp, calTree=calTree[i], orbitEphem=orbitEphem, horizonsProduct=horizonsProduct, copy=1)
            # The following task moves the SSO target to a fixed position in the map 
            if i == 0:
              timeOffset = frames.refs[0].product.getStatus("FINETIME")[0]
            frames = correctRaDec4Sso(frames , timeOffset=timeOffset, orbitEphem=orbitEphem, horizonsProduct=horizonsProduct, linear=0)
            # Save the frames to the ObservationContext 
            obs[i] = updatePacsObservation(obs[i], 1.0 , frames)
          # Remove variables that are not needed anymore
          del frames, pp, orbitEphem, horizonsProduct, i, timeOffset
        
        """
        1. First pass generating a mask per obsid using a soft threshold limit of S/N=3
        """
        
        for i in range(len(obsids)):
          print "Start processing: ", obsids[i]
          if camera == "blue":
            frames = obs[i].level1.refs["HPPAVGB"].product.refs[0].product
            if (frames.meta["blue"].getValue() == "blue1"):
                 lam = 70.0
                 filter="blue"
                 outpixsz = 1.1
                 hpfradius= 8
                 fwhm = 5.6
                 pixfrac = 1.0
            else:
                 lam = 100.0
                 filter="green"
                 outpixsz = 1.4
                 hpfradius= 9 
                 fwhm = 6.8
                 pixfrac = 1.0
          else:
            frames = obs[i].level1.refs["HPPAVGR"].product.refs[0].product
            lam = 160.0
            filter="red"
            outpixsz = 2.1
            hpfradius= 16 
            fwhm = 11.3
            pixfrac = 1.0
          fileRoot = direc + object + "_" + filter
          #
          # Define the sliding (half-)width to use for the highpass filtering
          #if camera == "blue":
          #    hpfradius = 15 
          #else:
          #    hpfradius = 25 
          # Calculate the coordinates of every pixel with time 
          if calculateRaDec: 
            frames = photAssignRaDec(frames)
          # Save the frames variable at its current state so we can use it later
          save(fileRoot + "_" + str(obsids[i]) + "_framesWithoutHPF.ser", "frames")
          #
          # First pass with highpass filter 
          #
          frames = highpassFilter(frames, hpfradius, interpolateMaskedValues=True)
          #frames = filterOnScanSpeed(frames, lowScanSpeed=lowScanSpeed, highScanSpeed=highScanSpeed)
          frames = filterOnScanSpeed(frames, limit=limits)
          map, mi = photProject(frames, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree[i])
          ##d = Display(map, title="HighpassFilter task result on " + str(obsids[i]) + " (without source mask)")
          if saveIntermediateSteps :
            simpleFitsWriter(map, fileRoot + "_" + str(obsids[i]) + "_map_withoutMask_firstStep.fits")
          # Obtain an source mask from this initial map
          med = MEDIAN(map.coverage[map.coverage.where(map.coverage > 0.)])
          index = map.image.where((map.coverage > med) & (ABS(map.image) < 1e-2))
          signal_stdev = STDDEV(map.image[index])
          threshold = 3.0*signal_stdev
          print "Threshold for the source mask = ", threshold
          mask = map.copy()
          mask.image[mask.image.where(map.image > threshold)] = 1.0
          mask.image[mask.image.where(map.image < threshold)] = 0.0
          mask.image[mask.image.where(map.coverage < 0.5*med)] = 0.0
          ##d = Display(mask, title="Source mask for " + str(obsids[i]))
          if saveIntermediateSteps :
            simpleFitsWriter(mask, fileRoot + "_" + str(obsids[i]) + "_mask_firstStep.fits")
          #
          # Second pass with masked highpass filtering
          #
          restore(fileRoot + "_" + str(obsids[i]) + "_framesWithoutHPF.ser")
          # Introduce the mask in the frames and apply the highpass filtering
          frames, outMask = photReadMaskFromImage(frames, si=mask, maskname="HighpassMask", extendedMasking=True, calTree=calTree[i])
          frames = highpassFilter(frames, hpfradius, maskname="HighpassMask", interpolateMaskedValues=True)
          #frames = filterOnScanSpeed(frames, lowScanSpeed=lowScanSpeed, highScanSpeed=highScanSpeed)
          frames = filterOnScanSpeed(frames, limit=limits)
          map, mi = photProject(frames, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree[i])
          ##d = Display(map, title="HighpassFilter task result on " + str(obsids[i]) + " (with a source mask)")
          simpleFitsWriter(map, fileRoot + "_" + str(obsids[i]) + "_map_firstStep.fits")
        
        # Remove variables that are not needed anymore
        del i, frames, map, mi, med, index, signal_stdev, threshold, mask, outMask 
        
        
        """
        2- Second pass for masking: using the previous masked maps
        """
        # Mosaic the maps
        if(len(obsids) > 1):
          images = ArrayList()
          for i in range(len(obsids)):
            #ima = SimpleImage()
            #importImage(image=ima, filename=fileRoot + "_" + str(obsids[i]) + "_map_firstStep.fits")
            ima = simpleFitsReader(file=fileRoot + "_" + str(obsids[i]) + "_map_firstStep.fits")
            images.add(ima)
          mosaic = MosaicTask()(images=images, oversample=0)
          del images, i, ima
        else:
          mosaic = simpleFitsReader(file=fileRoot + "_" + str(obsids[0]) + "_map_firstStep.fits")
        ##d = Display(mosaic, title="Mosaic map")
        if saveIntermediateSteps :
          simpleFitsWriter(mosaic, fileRoot + "_mosaic_firstStep.fits")
        
        # Obtain a source mask using the mosaic map
        med = MEDIAN(mosaic.coverage[mosaic.coverage.where(mosaic.coverage > 0.)])
        index = mosaic.image.where((mosaic.coverage > med) & (ABS(mosaic.image) < 1e-2))
        signal_stdev = STDDEV(mosaic.image[index])
        threshold = 2.0*signal_stdev
        print "Threshold for the source mask = ", threshold
        mosaicMask = mosaic.copy()
        mosaicMask.image[mosaicMask.image.where(mosaic.image > threshold)] = 1.0
        mosaicMask.image[mosaicMask.image.where(mosaic.image < threshold)] = 0.0
        mosaicMask.image[mosaicMask.image.where(mosaic.coverage < 0.5*med)] = 0.0
        ##d = Display(mosaicMask, title="Source mask obtained using the mosaic map")
        if saveIntermediateSteps :
          simpleFitsWriter(mosaicMask, fileRoot + "_mosaicMask_firstStep.fits")
        
        # Remove variables that are not needed anymore
        del mosaic, med, index, signal_stdev, threshold 
        
        """
        # 2. Second pass with proper mask derived from the mosaic map, 
        #    re-starting from level 1 from saved frames
        #    now with 2nd level deglitching
        """
        for i in range(len(obsids)):
          print "Start processing: ", obsids[i]
          restore(fileRoot + "_" + str(obsids[i]) + "_framesWithoutHPF.ser")
          frames, outMask = photReadMaskFromImage(frames, si=mosaicMask, maskname="HighpassMask", extendedMasking=True, calTree=calTree[i])
          frames = highpassFilter(frames, hpfradius, maskname="HighpassMask", interpolateMaskedValues=True)
          # Apply 2nd level deglitching after highpass filtering
          if(doIIndLevelDeg):
            s = Sigclip(10, 15) # nsigma=15 (second parameter) for sigma clipping
            s.behavior = Sigclip.CLIP
            s.outliers = Sigclip.BOTH_OUTLIERS
            s.mode = Sigclip.MEDIAN
            mapDeglitch(frames, algo=s, deglitchvector="timeordered", calTree=calTree[i])
            del s
          #frames = filterOnScanSpeed(frames, lowScanSpeed=lowScanSpeed, highScanSpeed=highScanSpeed)
          frames = filterOnScanSpeed(frames, limit=limits)
          map, mi = photProject(frames, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree[i])
          ##d = Display(map, title="HighpassFilter task result on " + str(obsids[i]) + " (with the mosaic source mask)")
          simpleFitsWriter(map, fileRoot + "_" + str(obsids[i]) + "_map_secondStep.fits")
          # Save the glitch mask into the non highpass filtered frames and save them again for the next step
          glitchmaskMeta = frames.meta["Glitchmask"]
          glitchmask = frames.getMask(glitchmaskMeta.value)
          restore(fileRoot + "_" + str(obsids[i]) + "_framesWithoutHPF.ser")
          frames.setMask(glitchmaskMeta.value, glitchmask)
          save(fileRoot + "_" + str(obsids[i]) + "_framesWithoutHPF.ser", "frames")
        
        # Remove variables that are not needed anymore
        del i, frames, outMask, map, mi,  glitchmaskMeta,  glitchmask
        
        """
        Mosaic the 2nd pass maps:
        """
        if(len(obsids) > 1):
          images = ArrayList()
          for i in range(len(obsids)):
            ima = simpleFitsReader(file=fileRoot + "_" + str(obsids[i]) + "_map_secondStep.fits")
            images.add(ima)
          mosaic = MosaicTask()(images=images, oversample=0)
          del images, i, ima
        else:
          mosaic = simpleFitsReader(file=fileRoot + "_" + str(obsids[0]) + "_map_secondStep.fits")
        ##d = Display(mosaic, title="Mosaic map")
        if saveIntermediateSteps :
          simpleFitsWriter(mosaic, fileRoot + "_mosaic_secondStep.fits")
        
        # Obtain a new source mask using the mosaic map
        med = MEDIAN(mosaic.coverage[mosaic.coverage.where(mosaic.coverage > 0.)])
        index = mosaic.image.where((mosaic.coverage > med) & (ABS(mosaic.image) < 1e-2))
        signal_stdev = STDDEV(mosaic.image[index])
        threshold = 2.0*signal_stdev
        print "Threshold for the source mask = ", threshold
        mosaicMask = mosaic.copy()
        mosaicMask.image[mosaicMask.image.where(mosaic.image > threshold)] = 1.0
        mosaicMask.image[mosaicMask.image.where(mosaic.image < threshold)] = 0.0
        mosaicMask.image[mosaicMask.image.where(mosaic.coverage < 0.5*med)] = 0.0
        ##d = Display(mosaicMask, title="Source mask obtained using the mosaic map")
        if saveIntermediateSteps :
          simpleFitsWriter(mosaicMask, fileRoot + "_mosaicMask_secondStep.fits")
        
        # Remove variables that are not needed anymore
        del  med, index, signal_stdev, threshold 
        
        """
        ################################################################################
        #  
        # 3. generates a mask with a circular patch centered on the source
        #    full source masking r=20arcsec
        """
        
        # Get the target coordinates
        if(doSourceFit):
          if(fromAFile):
            tlist, ralist, declist = readTargetList(tfile)
            # Loop over the targets, fit a gaussian to them, and save the fit coordinates
            rasource = Double1d(len(tlist))
            decsource = Double1d(len(tlist))
            for i in range(len(tlist)):
              pixcoor = mosaic.wcs.getPixelCoordinates(ralist[i], declist[i])
              cropsize = 20.
              r1 = int(pixcoor[0]-cropsize/2.)
              r2 = int(pixcoor[0]+cropsize/2.)
              c1 = int(pixcoor[1]-cropsize/2.)
              c2 = int(pixcoor[1]+cropsize/2.)  
              cmap = crop(image=mosaic, row1=int(pixcoor[0]-cropsize/2.),    \
                                        row2=int(pixcoor[0]+cropsize/2.),    \
                                        column1=int(pixcoor[1]-cropsize/2.), \
                                        column2=int(pixcoor[1]+cropsize/2.))
              try:
                sfit = mapSourceFitter(cmap)
                rasource[i] = Double(sfit["Parameters"].data[1])
                decsource[i] = Double(sfit["Parameters"].data[2])
                del sfit
              except:
                print "mapSourceFitter couldn't fit the source"
                print "The coordinates in the target file will be used"
                rasource[i] = ralist[i]
                decsource[i] = declist[i]
            #
            del ralist, declist, i, pixcoor, cropsize, r1, r2, c1, c2, cmap
          else:
            sfit = mapSourceFitter(mosaic)
            tlist = String1d(1, object)
            rasource = Double1d(1, sfit["Parameters"].data[1])
            decsource = Double1d(1, sfit["Parameters"].data[2])
            del sfit
        else:
          if(fromAFile):
            tlist, ralist, declist = readTargetList(tfile)
            rasource = Double1d(ralist)
            decsource = Double1d(declist)
            del ralist, declist
          else:
            # Get source Ra and Dec from the metadata
            tlist = String1d(1, obs[0].meta["object"].value)
            rasource  = Double1d(1, obs[0].meta["ra"].value)
            decsource = Double1d(1, obs[0].meta["dec"].value)
        
        for i in range(len(rasource)):
          print "Source = " + tlist[i] + ", coordinates = ", rasource[i], decsource[i]
        
        # Combine the the targets mask with the mosaic source mask
        radius = 20.
        combinedMask = mosaicMask.copy()
        #combinedMask.image = combinedMask.image*0. # uncomment if source list mask only
        mfc = MaskFromCatalogueTask()
        combinedMask = mfc(combinedMask, rasource, decsource, Double1d(len(rasource), radius), copy=1)
        ##d = Display(combinedMask, title="Combined mask")
        if saveIntermediateSteps :
          simpleFitsWriter(combinedMask, fileRoot + "_finalMosaicMask.fits")
        
        # Remove variables that are not needed anymore
        del mosaic, i, radius, mfc, mosaicMask
        
        """
        Final maps using fully masks and deglitched frames
        """
        for i in range(len(obsids)):
          print "Start processing: ", obsids[i]
          restore(fileRoot + "_" + str(obsids[i]) + "_framesWithoutHPF.ser")
          frames, outMask = photReadMaskFromImage(frames, si=combinedMask, maskname="HighpassMask", extendedMasking=True, calTree=calTree[i])
          frames = highpassFilter(frames, hpfradius, maskname="HighpassMask", interpolateMaskedValues=True)
          #frames = filterOnScanSpeed(frames, lowScanSpeed=lowScanSpeed, highScanSpeed=highScanSpeed)
          frames = filterOnScanSpeed(frames, limit=limits)
          # Make a map with the default pixfrac (=1)
          map, mi = photProject(frames, pixfrac=1, outputPixelsize=outpixsz, calTree=calTree[i])
          ##d = Display(map, title="Final map of " + str(obsids[i]) + " (pixfrac = 1)")
          simpleFitsWriter(map, fileRoot + "_" + str(obsids[i]) + "_finalMap_defaultPixfrac.fits")
          # Make a map with the user defined pixfrac
          map, mi = photProject(frames, pixfrac=pixfrac, outputPixelsize=outpixsz, calTree=calTree[i])
          ##d = Display(map, title="Final map of " + str(obsids[i]) + " (pixfrac = " + str(pixfrac) + ")")
          simpleFitsWriter(map, fileRoot + "_" + str(obsids[i]) + "_finalMap.fits")
        
        # Remove variables that are not needed anymore
        del i, frames, outMask, map, mi,  combinedMask
        
        """ 
        Mosaic the final maps:
        """
        if(len(obsids) > 1):
          images = ArrayList()  
          for i in range(len(obsids)):
            ima = simpleFitsReader(file=fileRoot + "_" + str(obsids[i]) + "_finalMap_defaultPixfrac.fits")
            images.add(ima)
          mosaicDefPixfrac = MosaicTask()(images=images, oversample=0)
          mosaicDefPixfrac.meta = images[0].meta
          mosaicDefPixfrac.history = images[0].history
          del images, i, ima
        else:
          mosaicDefPixfrac = simpleFitsReader(file=fileRoot + "_" + str(obsids[0]) + "_finalMap_defaultPixfrac.fits")
        ##d = Display(mosaicDefPixfrac, title="Final mosaic map (pixfrac = 1)")
        simpleFitsWriter(mosaicDefPixfrac, fileRoot + "_finalMosaic_defaultPixfrac.fits")
        
        if(len(obsids) > 1):
          images = ArrayList()
          for i in range(len(obsids)):
            ima = simpleFitsReader(file=fileRoot + "_" + str(obsids[i]) + "_finalMap.fits")
            images.add(ima)
          mosaic = MosaicTask()(images=images, oversample=0)
          mosaic.meta = images[0].meta
          mosaic.history = images[0].history
          del images, i, ima
        else:
          mosaic = simpleFitsReader(file=fileRoot + "_" + str(obsids[0]) + "_finalMap.fits")
        ##d = Display(mosaic, title="Final mosaic map (pixfrac = " + str(pixfrac) + ")")
        simpleFitsWriter(mosaic, fileRoot + "_finalMosaic.fits")
        
        # Remove variables that are not needed anymore
       
        
        """
        ############################################################################
        # photometry
        """
        if(doPhotometry):
          ap_blue  = [12,35,45]
          ap_green = [12,35,45]
          ap_red   = [11,18,23]
          # Obtain the filter and the aperture  
          filter = "red"
          aper_radius = ap_red
          if camera == "blue":
            if(obs[0].meta["blue"].value == "blue1"):
              filter = "blue"
              aper_radius = ap_blue
            if(obs[0].meta["blue"].value == "blue2"):
              filter = "green"
              aper_radius = ap_green
          print " Aperture used: " + str(aper_radius)
          del ap_blue, ap_green, ap_red
          # Do the photometry only for each source 
          fluxcorr = Double1d(len(rasource))
          fluxcorrbsub = Double1d(len(rasource))
          median =  Double1d(len(rasource))
          dev = Double1d(len(rasource))
          nBlankApertures =  Int1d(len(rasource))
          rms = Double1d(len(rasource))
          stn = Double1d(len(rasource))
          ##d = Display(mosaic, title="Aperture footprint")
          ##d.setCutLevelsPercentage(99.0)
          for source in range(len(rasource)):
            cenx = mosaic.getWcs().getPixelCoordinates(rasource[source], decsource[source])[1]
            ceny = mosaic.getWcs().getPixelCoordinates(rasource[source], decsource[source])[0]
            apphot = annularSkyAperturePhotometry(image=mosaic, centerX=cenx, centerY=ceny, \
                     radiusPixels=aper_radius[0], innerPixels=aper_radius[1], outerPixels=aper_radius[2], fractional=1)
            ##d.addEllipse(ceny, cenx, aper_radius[0]*2, aper_radius[0]*2, 1.0, java.awt.Color(0, 0, 255))
            ##d.addEllipse(ceny, cenx, aper_radius[1]*2, aper_radius[1]*2, 1.0, java.awt.Color(0, 0, 255))
            ##d.addEllipse(ceny, cenx, aper_radius[2]*2, aper_radius[2]*2, 1.0, java.awt.Color(0, 0, 255))
            # Correct the aperture photometry (use the calTree of the first observation)
            apphot_apcorr = photApertureCorrectionPointSource(apphot, responsivityVersion=7, band=filter, calTree=calTree[0])
            fluxcorr[source] = apphot_apcorr["Results table"]["Total flux"].data[0]
            fluxcorrbsub[source] = apphot_apcorr["Results table"]["Total flux"].data[2]
            ############################################################################
            # 1-sigma error estimate by placing blank sky apertures on the sky
            #
            step = 10. # in pixels
            xcorner = cenx-3*step  
            ycorner = ceny-3*step
            nx = 7
            ny = 7
            # Loop over the different positions
            fluxes = Double1d(nx*ny)
            counter = 0
            ##ed = Display(mosaic, title="Aperture photometry error estimation for" + tlist[source])
            ##ed.setCutLevelsPercentage(80.0)
            ##ed.zoomFactor = 2
            for i in range(nx):
              for j in range(ny):
                centerx = xcorner+step*i
                centery = ycorner+step*j
                try:
                  result = annularSkyAperturePhotometry(image=mosaic, centerX=centerx, centerY=centery,\
                           radiusPixels=aper_radius[0], innerPixels=aper_radius[1], outerPixels=aper_radius[2], fractional=1)
                  apphot = photApertureCorrectionPointSource(apphot=result, band=filter, responsivityVersion=7, calTree=calTree[0])
                  fluxes[counter] = apphot["Results table"]["Total flux"].data[0]
                  ##ed.addEllipse(centery, centerx, 2*aper_radius[0], 2*aper_radius[0], 1.0, java.awt.Color(255, 0, 0))
                except:
                  print "annularSkyAperturePhotometry crashed is field (" + str(i) + "," + str(j) + ") for " + tlist[source]
                  fluxes[counter] = 0;
                counter = counter + 1
            validFluxes = fluxes[fluxes.where(fluxes != 0)]
            median[source] = MEDIAN(validFluxes)
            dev[source] = validFluxes.apply(MedianAbsoluteDeviation(median[source]))
            index = fluxes.where(Bool1d(fluxes < median[source] + 2*dev[source]).and(Bool1d(fluxes != 0)))
            fluxBlankApertures = fluxes[index]
            nBlankApertures[source] = len(fluxBlankApertures)
            rms[source] = STDDEV(fluxBlankApertures)
            stn[source] = fluxcorr[source]/rms[source]
            # Visualize the blank sky apertures on the map
            n_aper = Int1d(nx*ny)
            n_aper[index] = 1
            counter = 0
            for i in range(nx):
              for j in range(ny):
                centerx = xcorner + step*i
                centery = ycorner + step*j
                if n_aper[counter] == 1:
                  counter=counter
                  ##ed.addEllipse(centery, centerx, 2*aper_radius[0], 2*aper_radius[0], 2, java.awt.Color(0, 0, 255))
                counter =  counter + 1
            # Plot the measured fluxes in the blank sky apertures
            ##p = PlotXY(fluxBlankApertures*1000)
            ##p.title.text = "Fluxes in blank sky apertures around "  + tlist[source]
            ##p.xaxis.title.text = "aperture"
            ##p.yaxis.title.text = "Flux (mJy)"
            ##p.style.line = 0
          # Print the results

          for source in range(len(rasource)):
            print >> file, " "
            print >> file,"-- Photometry results for " + tlist[source] + " --"
            print >> file,"ra , dec = ", rasource[source], " , ", decsource[source]
            print >> file,"Source flux (no background subtraction) = ", fluxcorr[source]*1000., "mJy"
            print >> file,"Source flux (with annular background subtraction) : ", fluxcorrbsub[source]*1000., "mJy"
            print >> file,"-- Error estimation --"
            print >> file,"Median = ", median[source]*1000., "mJy (including all apertures)"
            print >> file,"Median absolute deviation = ", dev[source]*1000., "mJy (including all apertures)"
            print >> file, "Number of blank sky apertures = ", nBlankApertures[source]
            print >> file, "STDDEV on blank sky apertures = ", rms[source]*1000., "mJy"
            print >> file,"Detection S/N = ", stn[source]
            print >> file," "
          # Delete some variables
          del aper_radius, apphot_apcorr, filter, cenx, ceny, step, xcorner, ycorner, nx, ny, counter,
          del  centerx, centery, result, apphot, fluxes, validFluxes, index, fluxBlankApertures, 
          del n_aper,  i, j, source
        
        # Remove files from the disk
        if not(saveIntermediateSteps):
          for i in range(len(obsids)):
            os.remove(fileRoot + "_" + str(obsids[i]) + "_framesWithoutHPF.ser")
            os.remove(fileRoot + "_" + str(obsids[i]) + "_map_firstStep.fits")
            os.remove(fileRoot + "_" + str(obsids[i]) + "_map_secondStep.fits")
        ### END
file.close()